"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.inValueRange = void 0;
const lodash_1 = require("lodash");
const bson_equal_1 = require("./bson-equal");
const inValueRange = (field, d) => {
    const compOperators = {
        $gte: function (a, b) {
            return a >= b;
        },
        $gt: function (a, b) {
            return a > b;
        },
        $lte: function (a, b) {
            return a <= b;
        },
        $lt: function (a, b) {
            return a < b;
        },
        $exists: function (a, b) {
            return a || b;
        },
        $eq: function (a, b) {
            return a === b;
        },
    };
    const conditions = [];
    const edgeCase = [];
    if (!(0, lodash_1.isPlainObject)(field)) {
        conditions.push(function (a) {
            return (0, lodash_1.isEqualWith)(a, field, bson_equal_1.bsonEqual);
        });
        edgeCase.push(field);
    }
    else {
        (0, lodash_1.forOwn)(field, function (value, key) {
            const comparator = compOperators[key];
            if (comparator) {
                conditions.push((0, lodash_1.curryRight)(comparator)(value));
                edgeCase.push(value);
            }
        });
    }
    const dx = (0, lodash_1.get)(d, 'dx', null);
    const bounds = dx ? (0, lodash_1.uniq)([d.value, d.value + dx]) : [d.bson || d.value];
    const results = (0, lodash_1.map)(bounds, function (bound, i) {
        if (i > 0) {
            bound -= 0.00001 * Math.abs(bound) + 0.00001;
        }
        return (0, lodash_1.every)((0, lodash_1.map)(conditions, function (cond) {
            return cond(bound);
        }));
    });
    if ((0, lodash_1.every)(results)) {
        return 'yes';
    }
    if ((0, lodash_1.some)(results)) {
        return 'partial';
    }
    if ((0, lodash_1.every)(edgeCase, function (val) {
        return val > bounds[0] && val < bounds[bounds.length - 1];
    })) {
        return 'partial';
    }
    return 'no';
};
exports.inValueRange = inValueRange;
//# sourceMappingURL=in-value-range.js.map